<?php
/**
 * This script simply checks if all translation files
 * are found in the messages folder
 * 
 * @author Florian Fackler <florian.fackler@mintao.com>
 * @version 0.01 Alpha
 * @license Do what you want with this file
 * 
 * // TODO: Generate autmoatically missing files and translations via google translate
 */
 
class CheckTranslationsCommand extends CConsoleCommand
{
    public function getHelp()
    {
        $commandLine = '';
        $argv = $_SERVER['argv'];
        foreach ($argv as $arg) {
            if ($arg == 'help') {
                continue;
            }
            $commandLine .= ' '.$arg;
        }
        return '
============================================================
Check for missing translation files or keys 
============================================================

This tool looks out for missing translation files contained
in your "messages" folder. Existing files will be checked
for missing words or phrases.

!! Append locales to ignore as arguments !!

____________________________________________________________
EXAMPLE CALL - check all but the french translations:

#> php ' . trim($commandLine) . ' fr_fr fr_ch

Missing *FILES* in folder de_de:
 > form.php
 > footer.php
Missing *FILES* in folder en_us:
 > headline.php


Missing *TRANSLATION KEY* in file "type.php" of de_de:
 > gold member
Missing *TRANSLATION KEY* in file "type.php" of en_us:
 > thank you
____________________________________________________________
';
    }
    
    public function run($args)
    {
        // Any languages to ignore?
        $ignore = count($args) > 0 ? $args : array();

        $folders = glob( Yii::getPathOfAlias('application.messages') . DIRECTORY_SEPARATOR . '*');
        if(count($folders)<=1) {
            throw new CException('Sure you already created a translation file?');
        }

        $allFiles = $allLocales = $localeFiles = $fileLocales = array();

        foreach ($folders as $folder) {
            $locale = basename($folder);
            if(in_array($locale, $ignore)) {
                continue;
            }
            $allLocales[] = $locale;
            $files = glob(realpath($folder) . DIRECTORY_SEPARATOR . '*.php');
            foreach ($files as $file) {
                $baseFile = basename($file);
                $allFiles[] = $baseFile;
                $localeFiles[$locale][$baseFile] = $file;
                $fileLocales[$baseFile][] = $locale;
            }
        }

        if(count($allLocales)<=1) {
            throw new CException('A minimum of 2 locales needed to find differences');
        }

        $allFiles = array_unique($allFiles);

        self::e('ln2');

        // Step 1: Check what files are missing
        $findCouter = 0;
        foreach ($localeFiles as $locale=>$files) {
            $missingFiles = array_diff($allFiles, array_keys($files));
            if (count($missingFiles)>0) {
                self::e("Missing *FILES* in folder \"$locale\":");
                foreach ($missingFiles as $file) {
                    self::e(" > $file");
                    ++$findCouter;
                }
            }
        }
        if ($findCouter == 0) {
            self::e('No missing files, congratulations!');
        }

        self::e('ln3' . str_repeat('-', 45) . 'ln1');
        self::e('ANALYZING ONLY FILES CONTAINED IN ALL FOLDERS');
        self::e('ln1' . str_repeat('-', 45) . 'ln3');
        
        // Step 2: check files contents
        $findCouter = 0;
        $countLocales = count($allLocales);
        foreach ($fileLocales as $file => $locales) {
            $localeTranslationKey = array();
            if($countLocales == count($locales)) {
                $allTranslationKeys = array();
                foreach($locales as $locale) {
                    $tmpArray = include $localeFiles[$locale][$file];
                    $localeTranslationKey[$locale][$file] = array_keys($tmpArray);
                    $allTranslationKeys = array_merge(
                        $allTranslationKeys,
                        $localeTranslationKey[$locale][$file]
                    );
                    unset($tmpArray);
                }

                $allTranslationKeys = array_unique($allTranslationKeys);

                foreach($locales as $locale) {
                    $missing = array_diff(
                        array_values($allTranslationKeys),
                        array_values($localeTranslationKey[$locale][$file])
                    );
                    if (count($missing) > 0) {
                        self::e('Missing *TRANSLATION KEY* in file "'.$file.'" of locale "'.$locale.'":');
                        foreach($missing as $translationKey) {
                            $translationKey = preg_replace('@\s+@',' ', $translationKey);
                            if (strlen($$translationKey) > 50) {
                                $translationKey = substr($translationKey, 0, 47) . '...';
                            }
                            self::e(" > $translationKey");
                            ++$findCouter;
                        }
                    }
                }
            }
        }
        if ($findCouter == 0) {
            self::e('No missing translations, congratulations!');
        }
        self::e('ln4');
    }

    /**
     * Simple output function with new line after each message.
     * To make extra linebreaks enter nl[COUNT] where [COUNT] is the number of new lines
     *
     * @param string $msg
     */
    public static function e($msg='')
    {
        $msg = preg_replace('@\bln(\d+)\b@e', 'str_repeat("\n", \\1)', $msg);
        echo "$msg\n";
    }
}
